// Forlix SoScore
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2008-2011 Dominik Friedrichs

// convar defaults
#define LOG_GOAL_SCORES           "1"
#define ANNOUNCEMENT_TYPE         "0"

#define SPAWN_MODE                "2"
#define RESPAWN_BLOCKTIME         "8"

#define OWN_GOAL_BAN_NUM          "3"
#define OWN_GOAL_BAN_TIME         "150"

#define PLAYER_GODMODE_HEALTH     "200"
#define TEAM_LIMIT                "8"

#define CHAT_TRIGGER              "!"

static Handle:h_log_goal_scores = INVALID_HANDLE;
static Handle:h_announcement_type = INVALID_HANDLE;

static Handle:h_spawn_mode = INVALID_HANDLE;
static Handle:h_respawn_blocktime = INVALID_HANDLE;

static Handle:h_own_goal_ban_num = INVALID_HANDLE;
static Handle:h_own_goal_ban_time = INVALID_HANDLE;

static Handle:h_player_godmode_health = INVALID_HANDLE;
static Handle:h_team_limit = INVALID_HANDLE;

static Handle:h_chat_trigger = INVALID_HANDLE;

static Handle:h_mp_restartgame = INVALID_HANDLE;
static Handle:h_sv_nomvp = INVALID_HANDLE;


SetupConVars()
{
  new Handle:version_cvar = CreateConVar(PLUGIN_VERSION_CVAR,
  PLUGIN_VERSION,
  "Forlix SoScore plugin version",
  FCVAR_PLUGIN|FCVAR_SPONLY|FCVAR_NOTIFY|FCVAR_PRINTABLEONLY);

  SetConVarString(version_cvar, PLUGIN_VERSION, false, false);

  h_log_goal_scores = CreateConVar("forlix_soscore_log_goal_scores",
  LOG_GOAL_SCORES,
  "Enables (1) or disables (0) goal score logging",
  0, true, 0.0, true, 1.0);

  h_announcement_type = CreateConVar("forlix_soscore_announcement_type",
  ANNOUNCEMENT_TYPE,
  "How often the plugin may announce itself to players: Every round-start (0), Once after connect (1)",
  0, true, 0.0, true, 1.0);

  // spawn cvars
  h_spawn_mode = CreateConVar("forlix_soscore_spawn_mode",
  SPAWN_MODE,
  "Disabled (0), Spawn new players after round-start (1), Respawn after death and team change as well (2)",
  0, true, 0.0, true, 2.0);

  h_respawn_blocktime = CreateConVar("forlix_soscore_respawn_blocktime",
  RESPAWN_BLOCKTIME,
  "Time in full seconds after death in which players are unable to respawn",
  0, true, 3.0, true, 30.0);

  // own goal cvars
  h_own_goal_ban_num = CreateConVar("forlix_soscore_own_goal_ban_num",
  OWN_GOAL_BAN_NUM,
  "Number of own goals required for a player to get banned (0 to disable)",
  0, true, 0.0, true, 20.0);

  h_own_goal_ban_time = CreateConVar("forlix_soscore_own_goal_ban_time",
  OWN_GOAL_BAN_TIME,
  "Time in minutes a player will be banned for (0 for permanent)",
  0, true, 0.0, true, 20160.0);

  // misc cvar
  h_player_godmode_health = CreateConVar("forlix_soscore_player_godmode_health",
  PLAYER_GODMODE_HEALTH,
  "Health value for players in godmode (0 to disable godmode and use default health)",
  0, true, 0.0, true, 250.0);

  h_team_limit = CreateConVar("forlix_soscore_team_limit",
  TEAM_LIMIT,
  "Limits the number of players on each team to the specified amount (0 to disable)",
  0, true, 0.0, true, 64.0);

  h_chat_trigger = CreateConVar("forlix_soscore_chat_trigger",
  CHAT_TRIGGER,
  "Set this to one of the ChatTrigger strings configured in SourceMod's core.cfg",
  FCVAR_PRINTABLEONLY);

  // up mp_roundtime convar bound to allow greater roundtimes
  SetConVarBounds(FindConVar("mp_roundtime"), ConVarBound_Upper, true, 90.0);

  h_mp_restartgame = FindConVar("mp_restartgame");
  h_sv_nomvp = FindConVar("sv_nomvp");

  HookConVarChange(h_log_goal_scores, MyConVarChanged);
  HookConVarChange(h_announcement_type, MyConVarChanged);

  HookConVarChange(h_spawn_mode, MyConVarChanged);
  HookConVarChange(h_respawn_blocktime, MyConVarChanged);

  HookConVarChange(h_own_goal_ban_num, MyConVarChanged);
  HookConVarChange(h_own_goal_ban_time, MyConVarChanged);

  HookConVarChange(h_player_godmode_health, MyConVarChanged);
  HookConVarChange(h_team_limit, MyConVarChanged);
  HookConVarChange(h_chat_trigger, MyConVarChanged);

  HookConVarChange(h_mp_restartgame, MyConVarChanged);
  HookConVarChange(h_sv_nomvp, MyConVarChanged);

  // manually trigger convar readout
  MyConVarChanged(INVALID_HANDLE, "0", "0");

  return;
}


public MyConVarChanged(Handle:convar,
                       const String:oldValue[],
                       const String:newValue[])
{
  log_goal_scores = GetConVarInt(h_log_goal_scores);
  announcement_type = GetConVarInt(h_announcement_type);

  spawn_mode = GetConVarInt(h_spawn_mode);
  respawn_blocktime = GetConVarInt(h_respawn_blocktime);

  own_goal_ban_num = GetConVarInt(h_own_goal_ban_num);
  own_goal_ban_time = GetConVarInt(h_own_goal_ban_time);

  player_godmode_health = GetConVarInt(h_player_godmode_health);
  team_limit = GetConVarInt(h_team_limit);

  GetConVarString(h_chat_trigger, help_cmd, sizeof(help_cmd));
  StrCat(help_cmd, sizeof(help_cmd), PLUGIN_COMMAND);

  // other cvars
  if(GetConVarInt(h_mp_restartgame))
    roundreset = true;

  sv_nomvp = GetConVarInt(h_sv_nomvp);

  // reset cvar depending stuff
  ResetPlayerHealth(false);

  return;
}
